import re

from zope.schema import TextLine
from zope.schema.interfaces import ValidationError

from collective.dnifield import _

KEY = 'TRWAGMYFPDXBNJZSQVHLCKE'


class WrongFormat(ValidationError):
    __doc__ = _("The entered value has a wrong format.")

class WrongLetter(ValidationError):
    __doc__ = _("The entered control letter doesn't match.")

class DNI(TextLine):
    """A field to enter a Spanish DNI.

    If an ending letter is provided, it will be validated against the
    entered number.
    """

    max_length = 9  # 8 digits + 1 letter = 9 chars

    def validate(self, value):
        """
        >>> dni = DNI()

        DNIs with a wrong format don't validate:
        >>> bad_formed = 'h4x0r'
        >>> dni.validate(bad_formed)
        Traceback (most recent call last):
        ...
        WrongFormat

        DNIs with the wrong letter don't validate:
        >>> bad_letter = '5892138Y'
        >>> dni.validate(bad_letter)
        Traceback (most recent call last):
        ...
        WrongLetter

        DNIs with the correct letter do validate:
        >>> good_w_letter = '0000000T'
        >>> dni.validate(good_w_letter)

        DNIs without letter but with a correct format do validate:
        >>> good_wo_letter = '0000000'
        >>> dni.validate(good_wo_letter)
        """
        match = re.match('^(\d{7,8})([A-z])?$', value) 
        if not match:
            raise WrongFormat

        number = int(match.group(1))
        letter = match.group(2)
        if letter and not KEY[number%23] == letter:
                raise WrongLetter



  # -*- extra stuff goes here -*- 

from zope.i18nmessageid import MessageFactory

_ = MessageFactory("collective.dnifield")

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

version = '1.0a1'

setup(name='collective.dnifield',
      version=version,
      description="A field to enter a national identity number.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        ],
      keywords='dni nif field',
      author='Israel Saeta Pérez',
      author_email='israel.saeta@dukebody.com',
      url='http://svn.plone.org/svn/collective/collective.dnifield',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'zope.schema',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import unittest
import doctest

def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        doctest.DocTestSuite('collective.dnifield.field',
                             optionflags=doctest.ELLIPSIS),
        ])
    return suite
